#ifndef CHROMEOS_CAMERA_LIB_GCAM_GCAM_AE_H_
#define CHROMEOS_CAMERA_LIB_GCAM_GCAM_AE_H_

#include <cstdint>
#include <memory>
#include <string>
#include <vector>

#if defined(CHROME_OS_CAMERA_BUILD_ENV)
#include "cros-camera/ae_stats_intel_ipu6.h"
#include "cros-camera/yuv_buffer.h"
#else
#include "chromeos/camera/lib/gcam/ae_stats_intel_ipu6.h"
#include "chromeos/camera/lib/gcam/yuv_buffer.h"
#endif

namespace cros {

// Copied from googlex/gcam/image_metadata/frame_metadata_constants.h.
constexpr float kLogSceneBrightnessUnknown = -999.0f;

// A normalized rectangle for a image with top-left (x0, y0) and bottom-right
// (x1, y1) coordinates in [0, 1].
struct NormalizedRect {
  float x0 = 0.0f;
  float x1 = 1.0f;
  float y0 = 0.0f;
  float y1 = 1.0f;

  float width() const { return x1 - x0; }
  float height() const { return y1 - y0; }
};

// AeResult carries the total exposure times (TETs) produced by the Gcam
// auto-exposure algorithm.
struct AeResult {
  // `short_tet` is the actual TET that should be applied to the sensor. Usually
  // we shorten the exposure time to avoid clipping the highlight areas.
  float short_tet = 0.0f;

  // `long_tet` is the desired exposure time for the shadow areas on the image,
  // and is used to produce the HDR ratio for the HDRnet pipeline.
  float long_tet = 0.0f;

  // `log_scene_brightness` is an absolute measure of the scene brightness
  // computed and used by Gcam AE. For information on the range of values and
  // what they mean, see comments in googlex/gcam/ae/ae_exemplar.h.
  float log_scene_brightness = kLogSceneBrightnessUnknown;

  bool IsValid() const { return short_tet > 0.0f && long_tet > 0.0f; }
};

// AeFrameMetadata encapsulates the capture metadata of a frame and carries the
// input parameters to the Gcam AE algorithm.
struct AeFrameMetadata {
  // The analog gain the frame was captured with.
  float actual_analog_gain = 0.0f;

  // The total digital gain applied on the frame, including sensor digital gains
  // and ISP digital gains.
  float applied_digital_gain = 0.0f;

  // The exposure time in ms the frame was captured with.
  float actual_exposure_time_ms = 0.0f;

  // The estimated sensor sensitivity of the camera computed with:
  //
  //   [sensitivity] = [min_iso] / ([f_number] ^ 2)
  float sensor_sensitivity = 0.0f;

  // Crop that was applied to generate the frame, in normalized
  // coordinates. The rectangle here shall corresponds to the settings in the
  // android.scaler.cropRegion metadata.
  NormalizedRect crop;

  // Face rectangles on the frame, in normalized coordinates. The coordinates
  // shall correspond to the values in the android.statistics.faceRectangles
  // metadata. The coordinates are normalized corresponding to the active sensor
  // area.
  std::vector<NormalizedRect> faces;

  // Metering areas. Used when, for example, the user taps the app window to do
  // touch AE. The coordinates shall correspond to the coordinates in the
  // android.control.aeRegions metadata.
  std::vector<NormalizedRect> metering_areas;

  // The exposure compensation in stops that shall be applied to the Gcam AE
  // output TETs.
  float exposure_compensation = 0.0f;
};

// AWB and color correction metadata.
struct AwbInfo {
  // (R, Gr, Gb, B) color channel gains following the same definition as the
  // Android camera metadata 'ANDROID_COLOR_CORRECTION_GAINS'.
  float gains[4] = {0};

  // 3x3 color correction matrix following the same definition as the Android
  // camera metadata 'ANDROID_COLOR_CORRECTION_TRANSFORM'.
  float ccm[9] = {0};
};

// Represents a TET range defined by [min_tet, max_tet].
struct TetRange {
  float min_tet = 0.0f;
  float max_tet = 0.0f;

  bool IsValid() const;
  std::string ToString() const;
};

class GcamAe {
 public:
  // Factory method to create a GcamAe instance.
  static std::unique_ptr<GcamAe> CreateInstance();

  // Computes a set of exposure parameters from the input `metadata`,
  // `yuv_buffer`, and `hdr_ratio`.  This is a somewhat generic and
  // device-agnostic approach that's good for initial evaluation of new
  // platforms.
  virtual AeResult ComputeLinearizedGcamAe(const AeFrameMetadata& metadata,
                                           const YuvBuffer& yuv_buffer,
                                           float max_hdr_ratio) = 0;

  // Computes exposure parameters through Gcam AE using Intel IPU6 AE stats.
  virtual AeResult ComputeGcamAe(int frame_width, int frame_height,
                                 const AeFrameMetadata& ae_metadata,
                                 const AwbInfo& awb_info,
                                 const AeStatsIntelIpu6& ae_stats_intel_ipu6,
                                 const TetRange& device_tet_range,
                                 float max_hdr_ratio) = 0;

  virtual ~GcamAe() = default;
};

}  // namespace cros

#endif  // CHROMEOS_CAMERA_LIB_GCAM_GCAM_AE_H_
