#ifndef CHROMEOS_CAMERA_LIB_GCAM_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_
#define CHROMEOS_CAMERA_LIB_GCAM_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_

#include <limits>
#include <memory>
#include <string>

#if defined(CHROME_OS_CAMERA_BUILD_ENV)
// clang-format off
#include <GLES3/gl3.h>
#include <GLES2/gl2ext.h>
// clang-format on
#include "cros-camera/texture_2d_descriptor.h"
#else
// clang-format off
#include "third_party/GL/gl/include/GLES3/gl3.h"
#include "third_party/GL/gl/include/GLES2/gl2ext.h"
// clang-format on
#include "chromeos/camera/lib/gcam/texture_2d_descriptor.h"
#endif

namespace cros {

// The interface for CrOS camera service to call into the HDRnet processing and
// rendering pipeline.
class HdrNetLinearRgbPipelineCrOS {
 public:
  // Initialization options for the HDRnet pipeline. For more details, see the
  // comments for hdrnet::gl::LinearRgbPipeline::CreateOptions in
  // google3/googlex/gcam/hdrnet/gl/linear_rgb_pipeline.h
  struct CreateOptions {
    // `input_width` and `input_height` are the size of the input texture.
    int input_width, input_height;
    // 'output_width and 'output_height` are the size of the output texture that
    // HDRnet renders into.
    int output_width, output_height;
    // `intermediate_format` is the GL format that will be used in the
    // intermediate texture inside the HDRnet pipeline.
    GLenum intermediate_format = GL_RGBA16F;
    // `min_hdr_ratio` and `max_hdr_ratio` are parameters needed by the HDRnet
    // pipeline.
    float min_hdr_ratio = 0.0f;
    float max_hdr_ratio = std::numeric_limits<float>::infinity();
  };

  // Run-time options for the HDRnet pipeline. For more details, see the
  // comments for hdrnet::gl::LinearRgbPipeline::RunOptions in
  // google3/googlex/gcam/hdrnet/gl/linear_rgb_pipeline.h
  struct RunOptions {
    // 'hdr_ratio' specifies the HDR ratio to use in this run.
    float hdr_ratio = -1.0f;

    // 'min_gain', 'max_gain' and 'max_gain_blend_threshold' defines the range
    // and threshold for HDR ratio interpolation. The HDR ratio is 1.0 when the
    // pixel luma is 0 and `max_gain` when luma greater than or equal to
    // `max_gain_blend_threshold`.
    float min_gain = 1.0f;
    float max_gain = std::numeric_limits<float>::infinity();
    float max_gain_blend_threshold = 0.0f;

    // `spatial_filter_sigma`, `range_filter_sigma` and 'iir_filter_strength`
    // are spatial and temporal filter parameters for smoothing out the LTM grid
    // the HDRnet model predicts.
    float spatial_filter_sigma = 0.0f;
    float range_filter_sigma = 0.0f;
    float iir_filter_strength = 0.0f;
  };

  // Factory method to create a LinearRgbPipelineCrOS instance. `options`
  // specifies the HDRnet pipeline configuration and will be passed to the
  // underlying hdrnet::gl::LinearRgbPipeline instance. `model_dir` specifies
  // the path on device to locate the HDRnet models. If `model_dir` is empty
  // string then the built-in model is used.
  static std::unique_ptr<HdrNetLinearRgbPipelineCrOS> CreatePipeline(
      const CreateOptions& options, const std::string model_dir = "");

  // Runs the HDRnet pipeline for coefficient prediction and HDRnet rendering.
  //
  // `input` is the input "raw" texture linearized from the 8-bit or 10-bit YUV
  // buffer produced by the ISP pipeline.
  //
  // `input_coeff_prediction` is an optional texture that will be used by the
  // HDRnet pipeline for coefficient prediction if presented. If
  // `input_coeff_prediction` is invalid then HDRnet pipeline will downsample
  // from the `input` texture to create the coefficient prediction texture
  // instead.
  //
  // `output` is the output texture that the HDRnet pipeline renders the result
  // into.
  //
  // `options` is the run-time options for the inference and rendering in this
  // run.
  virtual bool Run(Texture2DDescriptor input,
                   Texture2DDescriptor input_coeff_prediction,
                   Texture2DDescriptor output, const RunOptions& options) = 0;

  // Run the HDRnet pipeline customized for the Intel IPU6 ISP.
  //
  // The Intel IPU6 pipeline fuses the pre-processing and post-processing
  // required on the Intel IPU6 camera stack with the core HDRnet linear RGB
  // pipeline into one shader.
  //
  // The pre-processing converts the input NV12 texture `input_y` and `input_uv`
  // to linear RGB space by applying the `inverse_gamma_curve` and the
  // `inverse_adtm_curve`.
  //
  // The post-processing converts the rendered linear RGB texture back to YUV444
  // format in `output_y` and `output_uv` by applying the `adtm_curve` and the
  // `gamma_curve`.
  //
  // The pre-processing and the post-processing are combined with the linear RGB
  // pipeline in one shader program to minimize memory traffic and GPU sampling
  // counts.
  virtual bool RunIntelIpu6Pipeline(
      Texture2DDescriptor input_y, Texture2DDescriptor input_uv,
      Texture2DDescriptor inverse_gamma_curve,
      Texture2DDescriptor inverse_adtm_curve, Texture2DDescriptor gamma_curve,
      Texture2DDescriptor adtm_curve,
      Texture2DDescriptor input_coeff_prediction_rgb,
      Texture2DDescriptor output_y, Texture2DDescriptor output_uv,
      const RunOptions& options) = 0;

  virtual ~HdrNetLinearRgbPipelineCrOS() = default;
};

}  // namespace cros

#endif  // CHROMEOS_CAMERA_LIB_GCAM_HDRNET_LINEAR_RGB_PIPELINE_CROS_H_
