#ifndef CHROMEOS_CAMERA_LIB_GCAM_SPATIOTEMPORAL_DENOISER_H_
#define CHROMEOS_CAMERA_LIB_GCAM_SPATIOTEMPORAL_DENOISER_H_

#include <memory>
#include <vector>

#if defined(CHROME_OS_CAMERA_BUILD_ENV)
#include "cros-camera/texture_2d_descriptor.h"
#include "cros-camera/yuv_buffer.h"
#else
#include "chromeos/camera/lib/gcam/texture_2d_descriptor.h"
#include "chromeos/camera/lib/gcam/yuv_buffer.h"
#endif

namespace cros {

class SpatiotemporalDenoiser {
 public:
  enum class Mode {
    // Runs the denoiser on a burst of input YUV frames and produce one denoised
    // output frame. The denoiser runs denoising taking into consideration all
    // the input YUV frames.
    kBurstMode,

    // Runs the denoiser on one input of YUV frame and produce one denoised
    // output frame. All the input YUV frames are IIR-filtered and merged into
    // a temporal buffer. The denoiser runs temporal denoising using the
    // temporal buffer.
    kIirMode,
  };

  struct CreateOptions {
    // `frame_width` and `frame_height` are the width and height of the input
    // and output frames.
    int frame_width, frame_height;

    // By default the IIR temporal filter mode is used, as it's more efficient
    // for real-time video use-cases.
    Mode mode = Mode::kIirMode;
  };

  // Options for each denoising process run. For detailed description, see
  // http://cs/google3/googlex/gcam/cameralite/spatiotemporal_denoiser/spatiotemporal_denoiser_options.h.
  struct RunOptions {
    // The IIR filter strength of the temporal denoising passes. Values close to
    // 1.0 will prioritize on the accumulated frame, while value close to 0.0
    // will assign more weight to the input frame.
    float iir_temporal_convergence = 0.9f;

    // The filter strength multiplier for the spatial denoising passes. Higher
    // values will result in stronger denoising.
    float spatial_strength = 0.0f;

    // The number of spatial filtering passes to run. Setting the value to 0
    // will disable the spatial denoising pass.
    int num_spatial_passes = 0;

    // Whether to reset the accumulated buffer. Ususally set upon stream
    // (re-)configuration.
    bool reset_temporal_buffer = false;
  };

  // The SpatiotemporalDenoiser must be created and called on the same thread
  // with valid GL context.
  static std::unique_ptr<SpatiotemporalDenoiser> CreateInstance(
      const CreateOptions& options);

  virtual ~SpatiotemporalDenoiser() = default;

  // Runs the denoiser in burst mode on the set of `input_yuv_buffers` and write
  // the output to `output_yuv_buffer`. The denoiser instance must be
  // initialized with Mode::kBurstMode. Only NV12 YUV is supported at the
  // moment.
  virtual bool RunBurstDenoise(const std::vector<YuvBuffer>& input_yuv_buffers,
                               YuvBuffer output_yuv_buffer,
                               const RunOptions& options) = 0;

  // Runs the denoiser in IIR mode using the accumulated buffer from
  // `input_luma` and `input_chroma` over time, and write the output to
  // `output_luma` and `output_chroma`. The denoiser instance must be
  // initialized with Mode::kIirMode. Only NV12 YUV is supported at the moment.
  virtual bool RunIirDenoise(Texture2DDescriptor input_luma,
                             Texture2DDescriptor input_chroma,
                             Texture2DDescriptor output_luma,
                             Texture2DDescriptor output_chroma,
                             const RunOptions& options) = 0;
};

}  // namespace cros

#endif  // CHROMEOS_CAMERA_LIB_GCAM_SPATIOTEMPORAL_DENOISER_H_
